
#include <Arduino.h>
#include "Config.h"
#include "TimeClock.h"
#include <EEPROM.h>

#include "Input.h"
#include "Player.h"

#include <Snooze.h>

extern Conf conf;
extern elapsedMillis sleepTime;

// ********************************************************************************************************************************
// *******************************   Joystick  **************************************************************************************
// ********************************************************************************************************************************

extern Joystick  joyX,joyY ;      // declaration objet joystick

Joystick::Joystick(
                uint8_t pin,                             // pin 
                uint16_t neutral,                        // position neutre entre 0 et 1023
                uint8_t deadBand,    
                int8_t direction,                         // direction -1 ou 1
                uint16_t max,
                uint16_t min
                ) : _pin(pin),  _neutral(neutral), _min(min), _max(max), _deadBand(deadBand),  _direction(direction)
  {  
    pinMode(_pin, INPUT);
  }

int16_t Joystick::get() 
  { 
    _read = analogRead(_pin);
    int16_t _value;
    
    if (_read  > _max) _read = _max;
    if (_read  < _min) _read = _min;
  
    if (_read  > _neutral + _deadBand) _value=_direction  * map (_read, _neutral + _deadBand , _max, 0, 500);  // (_value-_deadBand);
    else if (_read < _neutral -_deadBand) _value = _direction  * map (_read, _min, _neutral -_deadBand, -500, 0);           // (_value+_deadBand);
         else _value=0;
    
    return _value; 
  }

uint16_t Joystick::rawGet()   {  return _read;  }

// ********************************************************************************************************************************
// *******************************   Switch  **************************************************************************************
// ********************************************************************************************************************************

boolean  Switch::_anyShortPress = LOW;  // pour n'importe quel bouton

uint32_t Switch::_lastPress = millis();
  
Switch::Switch(uint8_t initPin) 
  : _pin(initPin)
  {
  pinMode(initPin , INPUT_PULLUP);
  _type = PIN;
  }

Switch::Switch(Joystick *joy, int16_t th) 
    : _joy(joy), _th(th)  
    {  _type = JOY;  }

void Switch::begin() { _timePress=millis(); }

#define LONG_PRESS_DELAY  1500

void Switch::handle() 
    {  
    
      // reinitialisation des états de sorti des boutons
      _shortPress = LOW;   _longPress = LOW;
     
      _currentMillis = millis();
      
      if (_type==PIN) _state=!digitalRead(_pin);
      else if (_th > 0 ) _state = (_joy->get() > _th );
           else _state = (_joy->get() < _th );
    
      if (_state && !_lastState) { // on presse le bouton
          if (conf.enableSound) tone (5,80,20);
          _lastState = _state; 
          _timePress = _currentMillis;
          sleepTime = 0;
          
          _resetLongPress = HIGH;
          _lastPress = _currentMillis;
          
          _shortPress = HIGH;
          _anyShortPress = HIGH;      // Variable static pour savoir si n'importe quel bouton a été pressé
          }
      
      if (!_state && _lastState) // on relache le bouton
          { 
          _lastState = _state; 
          _resetLongPress = LOW;
          _timePress = _currentMillis;
          }
      
      if (_resetLongPress && _currentMillis-_timePress > LONG_PRESS_DELAY ) 
        {
            if (conf.enableSound) tone (5,120,40);
            _longPress = HIGH;
            _resetLongPress = LOW;
        } 
    }

boolean Switch::state()  { return _state; }

boolean Switch::shortPress() {  return _shortPress;  }
       
boolean Switch::longPress() {  return _longPress;  }

boolean Switch::anyShortPress()  { return _anyShortPress;  }

void Switch::resetAnyShortPress()   { _anyShortPress = LOW; }

// ********************************************************************************************************************************
// *******************************   Gyro **************************************************************************************
// ********************************************************************************************************************************

#define MPU6050_ADDRESS    0x69  // 
#define SIGNAL_PATH_RESET  0x68
#define I2C_SLV0_ADDR      0x37
#define ACCEL_CONFIG       0x1C 
#define MOT_THR            0x1F  // Motion detection threshold bits [7:0]
#define MOT_DUR            0x20  // Duration counter threshold for motion interrupt generation, 1 kHz rate, LSB = 1 ms
#define MOT_DETECT_CTRL    0x69
#define INT_ENABLE         0x38
#define WHO_AM_I_MPU6050   0x75 // Should return 0x68
#define INT_STATUS 0x3A
 
#define ACCELEROMETER_SENSITIVITY 8192.0
#define GYROSCOPE_SENSITIVITY 65.536

#define SCALE_FOR_MAP   1.0
  
  void Gyro::begin ()

{
    Wire.begin();
    initialize();
    //dmpInitialize();  // to do 
}
  
  void Gyro::handle()
        {
         getMotion6(&_ax, &_ay, &_az, &_gx, &_gy, &_gz);              
         
         if (abs(_gx) > 500 ) sleepTime = 0 ;
         
            //  OFFSET
            _gx += 130;
            
            _roll =  (atan2f((float)_ax, (float)_az) * 1024 / M_PI) ; // 
            _rollMapTarget = (int16_t)( _roll) >> 3 ;


          if (conf.enableGyro)
          {
            if (abs( _rollMap  - _rollMapTarget) > 4 ) 
              {
                if ( _rollMap < _rollMapTarget) _rollMap+=2;
                else  _rollMap-=2;
              }
          }
          else   _rollMap = 0;
          

            if (!conf.lockScreen)
              {
                 if ( _rollMapTarget < -40  && abs(_ay) < 10000 ) conf.screenSens = 1 ;
                 if ( _rollMapTarget >  40  && abs(_ay) < 10000) conf.screenSens = 0 ;
              }
         
         }
         
int8_t Gyro::getRollMap()
        {
           return _rollMap ;
        }

int16_t Gyro::gx()
        {
           return _gx;
        }

int16_t Gyro::ax()
        {
           return _ax;
        }
        
//#endif
